"""Data models for API responses."""

from pydantic import BaseModel, ConfigDict, Field, model_validator

class Track(BaseModel):
    """Represents a single track."""

    model_config = ConfigDict(populate_by_name=True)

    start_time: str | None = Field(None, alias="startTime")
    end_time: str | None = Field(None, alias="endTime")
    artist: str | None = None
    title: str | None = None
    label: str | None = None


class Tracklist(BaseModel):
    """Represents a tracklist (audiostream)."""

    model_config = ConfigDict(populate_by_name=True)

    name: str | None = None
    slug: str | None = None
    url: str | None = None
    duration: str | None = None
    tracks: list[Track] | None = None

    @model_validator(mode="before")
    @classmethod
    def populate_tracks_from_detection_processes(cls, data):
        """Populate tracks field from detectionProcesses if tracks is None."""
        if isinstance(data, dict):
            tracks = data.get("tracks")
            detection_processes = data.get("detectionProcesses")

            if tracks is None and detection_processes:
                all_tracks = []
                for process in detection_processes:
                    if process.get("detectionProcessMusicTracks"):
                        all_tracks.extend(process["detectionProcessMusicTracks"])
                if all_tracks:
                    data = data.copy()
                    data["tracks"] = all_tracks
        return data


class SearchResult(BaseModel):
    """Represents search results."""

    results: list[Tracklist] = Field(alias="audiostreams")
    page: int | None = None
    per_page: int | None = None



"""Main API client for TrackID.net."""

import httpx

from trackidnet_client.models import Tracklist, SearchResult


class TrackIDNet:
    """Client for interacting with the TrackID.net API."""

    def __init__(
        self,
        base_url: str = "https://trackid.net:8001/api/public",
        timeout: float = 5.0,
        headers: dict[str, str] | None = None,
    ):
        self.base_url = base_url
        self.timeout = timeout
        self._default_headers = {
            "User-Agent": "trackidnet-client/0.1.0",
            "Accept": "*/*",
            "Origin": "https://trackid.net",
            "Referer": "https://trackid.net/",
        }
        if headers:
            self._default_headers.update(headers)

        self._client = httpx.Client(
            base_url=self.base_url,
            timeout=self.timeout,
            headers=self._default_headers,
        )

    def search_tracklist(
        self,
        keywords: str,
        current_page: int = 0,
        page_size: int = 20,
        **kwargs,
    ) -> SearchResult:
        """Search for tracklists"""
        params = {
            "keywords": keywords,
            "currentPage": current_page,
            "pageSize": page_size,
        }
        params.update(kwargs)

        data = self._client.get("/audiostreams", params=params)
        results = data.json().get("result")
        return SearchResult.model_validate(results)

    def get_tracklist(self, slug: str) -> Tracklist:
        """Get a specific tracklist by slug."""
        
        data = self._client.get(f"/audiostreams/{slug}")
        result = data.json().get("result")
        return Tracklist.model_validate(result)

